import sys
import time
import pyprind


n = 100
sleeptime = 0.02


def test_bar():
    bar = pyprind.ProgBar(n)
    for i in range(n):
        time.sleep(sleeptime)
        bar.update()


def test_stdout():
    bar = pyprind.ProgBar(n, stream=sys.stdout)
    for i in range(n):
        time.sleep(sleeptime)
        bar.update()


def test_generator():
    for i in pyprind.prog_bar(range(n), stream=sys.stdout):
        time.sleep(sleeptime)


def test_monitoring():
    bar = pyprind.ProgBar(n, monitor=True)
    for i in range(n):
        time.sleep(sleeptime)
        bar.update()
    print(bar)


def test_width():
    bar = pyprind.ProgBar(n, width=10)
    for i in range(n):
        time.sleep(sleeptime)
        bar.update()


def test_item_tracking():
    items = ['file_%s.csv' % i for i in range(0, n)]
    bar = pyprind.ProgBar(len(items))
    for i in items:
        time.sleep(sleeptime)
        bar.update(item_id=i)


def test_character():
    bar = pyprind.ProgBar(n, bar_char='>')
    for i in range(n):
        time.sleep(sleeptime)
        bar.update()


def test_force_flush():
    bar = pyprind.ProgBar(n)
    for i in range(n):
        time.sleep(sleeptime)
        bar.update(force_flush=True)


def test_update_interval():
    bar = pyprind.ProgBar(n, update_interval=0.1)
    for i in range(n):
        time.sleep(sleeptime)
        bar.update()


if __name__ == '__main__':

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Basic Progress Bar\n')
    test_bar()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing stdout Stream\n')
    test_stdout()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Progress Bar Generator\n')
    test_generator()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing monitor function\n')
    test_monitoring()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Width Parameter\n')
    test_width()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Item Tracking\n')
    test_item_tracking()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print("Testing Progress Bar Character ('>', default: '#')\n")
    test_character()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Force Flush\n')
    test_force_flush()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Update Interval\n')
    test_update_interval()



"""
Sebastian Raschka 2014-2017
Python Progress Indicator Utility

Author: Sebastian Raschka <sebastianraschka.com>
License: BSD 3 clause

Contributors: https://github.com/rasbt/pyprind/graphs/contributors
Code Repository: https://github.com/rasbt/pyprind
PyPI: https://pypi.python.org/pypi/PyPrind
"""


from math import floor
from pyprind.prog_class import Prog
import time


class ProgBar(Prog):
    """
    Initializes a progress bar object that allows visuzalization
    of an iterational computation in the standard output screen.

    Parameters
    ----------
    iterations : `int`
        Number of iterations for the iterative computation.
    track_time : `bool` (default: `True`)
        Prints elapsed time when loop has finished.
    width : `int` (default: 30)
        Sets the progress bar width in characters.
    stream : `int` (default: 2).
        Setting the output stream.
        Takes `1` for stdout, `2` for stderr, or a custom stream object
    title : `str` (default:  `''`)
        Setting a title for the progress bar.
    monitor : `bool` (default: False)
        Monitors CPU and memory usage if `True` (requires `psutil` package).
    update_interval : float or int (default: None)
        The update_interval in seconds controls how often the progress
        is flushed to the screen.
        Automatic mode if update_interval=None.

    """
    def __init__(self, iterations, track_time=True, width=30, bar_char='#',
                 stream=2, title='', monitor=False, update_interval=None):
        Prog.__init__(self, iterations, track_time,
                      stream, title, monitor, update_interval)
        self.bar_width = width
        self._adjust_width()
        self.bar_char = bar_char
        self.last_progress = 0

        if monitor:
            try:
                self.process.cpu_percent()
                self.process.memory_percent()
            except AttributeError:  # old version of psutil
                self.process.get_cpu_percent()
                self.process.get_memory_percent()
        if self.item_id:
            self._cache_item_id()

    def _adjust_width(self):
        """Shrinks bar if number of iterations is less than the bar width"""
        if self.bar_width > self.max_iter:
            self.bar_width = int(self.max_iter)
            # some Python 3.3.3 users specifically
            # on Linux Red Hat 4.4.7-1, GCC v. 4.4.7
            # reported that self.max_iter was converted to
            # float. Thus this fix to prevent float multiplication of chars.

    def _cache_progress_bar(self, progress):
        remaining = self.bar_width - progress
        self._cached_output += '\r0% [{}{}] 100%'.format(self.bar_char * int(progress),
                                               ' ' * int(remaining))

    def _print(self, force_flush=False):
        progress = floor(self._calc_percent() / 100 * self.bar_width)
        if self.update_interval:
            do_update = time.time() - self.last_time >= self.update_interval
        elif force_flush:
            do_update = True
        else:
            do_update = progress > self.last_progress

        if do_update and self.active:

            self._cache_progress_bar(progress)
            if self.track:
                self._cache_eta()
            if self.item_id:
                self._cache_item_id()
            self._stream_out('%s' % self._cached_output)
            self._stream_flush()
            self._cached_output = ''
        self.last_progress = progress



"""
Sebastian Raschka 2014-2016
Python Progress Indicator Utility

Author: Sebastian Raschka <sebastianraschka.com>
License: BSD 3 clause

Contributors: https://github.com/rasbt/pyprind/graphs/contributors
Code Repository: https://github.com/rasbt/pyprind
PyPI: https://pypi.python.org/pypi/PyPrind
"""

import sys
import time
import pyprind

n = 100
sleeptime = 0.02


def test_basic_percent():
    perc = pyprind.ProgPercent(n)
    for i in range(n):
        time.sleep(sleeptime)
        perc.update()


def test_stdout():
    perc = pyprind.ProgPercent(n, stream=sys.stdout)
    for i in range(n):
        time.sleep(sleeptime)
        perc.update()


def test_generator():
    for i in pyprind.prog_percent(range(n), stream=sys.stdout):
        time.sleep(sleeptime)


def test_monitoring():
    perc = pyprind.ProgPercent(n, monitor=True)
    for i in range(n):
        time.sleep(sleeptime)
        perc.update()
    print(perc)


def test_item_tracking():
    items = ['file_%s.csv' % i for i in range(0, n)]
    perc = pyprind.ProgPercent(len(items))
    for i in items:
        time.sleep(sleeptime)
        perc.update(item_id=i)


def test_force_flush():
    perc = pyprind.ProgPercent(n)
    for i in range(n):
        time.sleep(sleeptime)
        perc.update(force_flush=True)


def test_update_interval():
    perc = pyprind.ProgPercent(n, update_interval=4)
    for i in range(n):
        time.sleep(sleeptime)
        perc.update()


if __name__ == "__main__":
    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Basic Percentage Indicator\n')
    test_basic_percent()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing stdout Stream\n')
    test_stdout()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Percentage Indicator Generator\n')
    test_generator()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing monitor function\n')
    test_monitoring()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Item Tracking\n')
    test_item_tracking()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Force Flush\n')
    test_force_flush()

    print('\n%s' % (80 * '='))
    print('%s\n' % (80 * '='))
    print('Testing Update Interval\n')
    test_update_interval()



"""
Sebastian Raschka 2014-2016
Python Progress Indicator Utility

Author: Sebastian Raschka <sebastianraschka.com>
License: BSD 3 clause

Contributors: https://github.com/rasbt/pyprind/graphs/contributors
Code Repository: https://github.com/rasbt/pyprind
PyPI: https://pypi.python.org/pypi/PyPrind
"""


from setuptools import setup, find_packages
import pyprind

VERSION = pyprind.__version__

setup(name='PyPrind',
      version=VERSION,
      description='Python Progress Bar and Percent Indicator Utility',
      author='Sebastian Raschka',
      author_email='mail@sebastianraschka.com',
      url='https://github.com/rasbt/pyprind',
      packages=find_packages(),
      package_data={'': ['LICENSE',
                         'README.md',
                         'requirements.txt',
                         'CHANGELOG.md',
                         'CONTRIBUTING.md'],
                    'tests': ['tests/test_percentage_indicator.py',
                              'tests/test_progress_bar.py']},
      include_package_data=True,
      license='BSD 3-Clause',
      platforms='any',
      classifiers=[
          'License :: OSI Approved :: BSD License',
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Environment :: Console'],
      long_description="""

The PyPrind (Python Progress Indicator) module provides a progress
bar and a percentage indicator
object that let you track the progress of a loop structure or
other iterative computation.
Typical applications include the processing of
large data sets to provide an intuitive estimate
at runtime about the progress of the computation.

For more details and examples please see the package documentation
https://github.com/rasbt/pyprind/blob/master/README.md.

Contact
=============

If you have any questions or comments about PyPrind,
please feel free to contact me via
email: mail@sebastianraschka.com
or Twitter: https://twitter.com/rasbt""",)
